* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=MAX;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; 

***********************************************************************************;
*******   MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR CHAPTER 10a MODELS    *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use FitTest macro;
* FitFewer = Name of ODS InfoCrit table for nested model;
* FitMore  = Name of ODS InfoCrit table for comparison model;
%MACRO FitTest(FitFewer=,FitMore=);
DATA &FitFewer.; LENGTH Name $30.; SET &FitFewer.; Name="&FitFewer."; RUN;
DATA &FitMore.;  LENGTH Name $30.; SET &FitMore.;  Name="&FitMore.";  RUN;
DATA FitCompare; LENGTH Name $30.; SET &FitFewer. &FitMore.; RUN;
DATA FitCompare; SET FitCompare; DevDiff=Lag1(Neg2LogLike)-Neg2LogLike;
     DFdiff=Parms-LAG1(Parms); Pvalue=1-PROBCHI(DevDiff,DFdiff);
     DROP AICC HQIC CAIC; RUN;
TITLE9 "Likelihood Ratio Test for &FitFewer. vs. &FitMore.";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE9;
%MEND FitTest;

* To use TotalR2 macro;
* DV =    Case-sensitive name of dependent variable (required);
* Pred1 = Name of OUTPM= data file of predicted outcomes for first model (required);
* Pred2 = Name of OUTPM= data file of predicted outcomes for second model (optional);
%MACRO TotalR2(DV=, Pred1=, Pred2=);
PROC CORR NOPRINT NOSIMPLE DATA=&Pred1. OUTP=Corr1; VAR pred &DV.; RUN;
DATA Corr1; LENGTH Name $30.; SET Corr1; Name="&Pred1.";
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
PROC CORR NOPRINT NOSIMPLE DATA=&Pred2. OUTP=Corr2; VAR pred &DV.; RUN;
DATA Corr2; LENGTH Name $30.; SET Corr2; Name="&Pred2."; 
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
DATA CorrCompare; LENGTH Name $30.; SET Corr1 Corr2; 
     TotalR2Diff=TotalR2-LAG1(TotalR2);
     KEEP Name PredCorr TotalR2 TotalR2Diff; RUN; 
%END;
%IF %LENGTH(&Pred2.)>0 %THEN %LET ToPrint=CorrCompare; 
     %ELSE %LET ToPrint=Corr1;
TITLE8 "Total-R2 Per Model";
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
TITLE9 "Change in Total-R2 for &Pred1. vs. &Pred2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;   
%MEND TotalR2;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested baseline model (required);
* CovMore1 = Name of ODS CovParms table for comparison model1 (required);
* CovMore2 = Name of ODS CovParms table for comparison model2 (optional);
%MACRO PseudoR2(NCov=, CovBase=, CovMore1=, CovMore2=);
DATA &CovBase.;   LENGTH Name $30.; SET &CovBase.;  Name="&CovBase.";  RUN;
DATA &CovMore1.;  LENGTH Name $30.; SET &CovMore1.; Name="&CovMore1."; RUN;
DATA CovCompare1; LENGTH Name $30.; SET &CovBase. &CovMore1.; RUN;
DATA CovCompare1; SET CovCompare1; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
%IF %LENGTH(&CovMore2.)>0 %THEN %DO; %LET lagchange=%EVAL(&Ncov.*2);
DATA &CovMore2.; LENGTH Name $30.; SET &CovMore2.; Name="&CovMore2."; RUN;
DATA CovCompare2; LENGTH Name $30.; SET &CovBase. &CovMore2.; RUN;
DATA CovCompare2; SET CovCompare2; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare1 CovCompare2;
     PseudoR2Change=PseudoR2-(LAG&lagchange(PseudoR2)); RUN;
%END; %IF %LENGTH(&CovMore2.)>0 %THEN %LET ToPrint=CovCompare; 
     %ELSE %LET ToPrint=CovCompare1;
DATA &ToPrint.; SET &ToPrint.; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(3,2)","UN(4,1)","UN(4,2)","UN(4,3)",
                   "UN(5,1)","UN(5,2)","UN(5,3)","UN(5,4)",
                   "UN(6,1)","UN(6,2)","UN(6,3)","UN(6,4)","UN(6,5)") THEN DELETE; 
     DROP Zvalue ProbZ; RUN;
TITLE8 "Pseudo-R2 Relative to &CovBase.";
%IF %LENGTH(&CovMore2.)>0 %THEN %DO;
TITLE9 "Change in Pseudo-R2 for &CovMore1. vs. &CovMore2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;
%MEND PseudoR2;


***********************************************************************************;
*******  BEGIN DATA MANIPULATION FOR CHAPTER 10a ALTERNATIVE TIME MODELS    *******;
*******                 CHANGE "filesave" to your directory                 *******;
***********************************************************************************;

* Defining global variable for file location to be replaced in code below;
%LET filesave= C:\Dropbox\25_PSQF7375_AdvLong\PSQF7375_AdvLong_Example3_MLM;
* Location for SAS files for these models (uses macro variable filesave);
LIBNAME filesave "&filesave.";

* Import chapter 10a stacked data into work library;
DATA work.Example3; SET filesave.SAS_Chapter10a; RUN;

* Data preparation;
DATA work.Example3; SET work.Example3;
* Create time in study (and squared version);
  time = tvage-ageT0;
  timesq = time*time;
* Fix 1 case rounded to 9;
  IF occasion=9 THEN occasion=8;
* Create rounded age (years since birth);
  roundage = ROUND(tvage,1);
* Fix 2 cases above 95;
  IF roundage>95 THEN roundage=95;
* Center TV age and age at time 0 at 84 (and squared versions);
  tvage84 = tvage-84;
  tvage84sq = tvage84*tvage84;
  ageT084 = ageT0-84;
  ageT084sq = ageT084*ageT084;
* Label variables;
LABEL
  time      = "time: Years since Time 0"
  timesq    = "timesq: Squared Years since Time 0"
  roundage  = "roundage: Age Rounded to Nearest Year"
  tvage84   = "tvage84: Time-Varying Age (0=84 years)"
  tvage84sq = "tvage84sq: Squared Time-Varying Age (0=84 years)"
  ageT084   = "ageT084: Age at Time 0 (0=84 years)"
  ageT084sq = "ageT084sq: Squared Age at Time 0 (0=84 years)";
* Subset sample to complete cases for all predictors;
  IF NMISS(tvage, ageT0, recall)>0 THEN DELETE;
RUN; 

***********************************************************************************;
*******      BEGIN DESCRIPTION VIA EMPTY AND SATURATED MEANS MODELS         *******;
***********************************************************************************;

* Open external file to save results to;
ODS RTF FILE="&filesave.\AdvLong_Example3_MLM_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE; 

TITLE1 "Descriptive Statistics";
PROC MEANS NDEC=4 DATA=work.Example3;
     VAR ageT0 tvage time recall;
RUN; 
PROC CORR NOSIMPLE DATA=work.Example3;
     VAR ageT0 tvage time recall;
RUN; TITLE1;


TITLE1 "Empty Means, Random Intercept Model for Age";
PROC MIXED DATA=work.Example3 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
     CLASS PersonID;
     MODEL tvage =  / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / VCORR TYPE=UN SUBJECT=PersonID;
RUN; TITLE1;
 
TITLE1 "Empty Means, Random Intercept Model for Time";
PROC MIXED DATA=work.Example3 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
     CLASS PersonID;
     MODEL time =  / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / VCORR TYPE=UN SUBJECT=PersonID;
RUN; TITLE1;

TITLE1 "Model 0: Empty Means, Random Intercept Model for Recall Outcome";
PROC MIXED DATA=work.Example3 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
     CLASS PersonID;
     MODEL recall =  / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / VCORR TYPE=UN SUBJECT=PersonID;
     ODS OUTPUT CovParms=CovEmpty; * Save for pseudo-R2;
RUN; TITLE1;
 

TITLE1 "Saturated Means by Rounded Age, Random Intercept Model";
PROC GLIMMIX DATA=work.Example3 NOCLPRINT NAMELEN=100 METHOD=MSPL;
     CLASS PersonID roundage;
     MODEL recall = roundage / LINK=IDENTITY DIST=NORMAL DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=PersonID;
     LSMEANS roundage / PLOT=MEANPLOT(JOIN CLBAND);
RUN; TITLE1;
 
TITLE1 "Saturated Means by Rounded Time, Random Intercept Model";
PROC GLIMMIX DATA=work.Example3 NOCLPRINT NAMELEN=100 METHOD=MSPL;
     CLASS PersonID occasion;
     MODEL recall = occasion / LINK=IDENTITY DIST=NORMAL DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=PersonID;
     LSMEANS occasion / PLOT=MEANPLOT(JOIN CLBAND);
RUN; TITLE1;

 
********************************************************************************;
******                     BEGIN AGE-AS-TIME MODELS                      *******;
********************************************************************************;

TITLE1 "Model 1a Age: Fixed Quadratic, Random Intercept Model";
PROC MIXED DATA=work.Example3 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
CLASS PersonID;
MODEL recall = tvage84 tvage84sq / SOLUTION DDFM=Satterthwaite OUTPM=PredRIAge; * Save for total R2;
RANDOM INTERCEPT / TYPE=UN SUBJECT=PersonID;
ODS OUTPUT InfoCrit=FitRIAge CovParms=CovRIAge; * Save for LRT and pseudo-R2;
RUN; TITLE1;
* Call macro to calculate Total R2;
  %TotalR2(DV=recall, Pred1=PredRIAge);
* Call macro to calculate Pseudo-R2;
  %PseudoR2(Ncov=2, CovBase=CovEmpty, CovMore1=CovRIAge);

TITLE1 "Model 2a Age: Fixed Quadratic, Random Intercept Model";
TITLE2 "Controlling for Birth Cohort as Contextual Effects";
PROC MIXED DATA=work.Example3 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
CLASS PersonID;
MODEL recall = tvage84 tvage84sq ageT084 ageT084sq tvage84*ageT084
                 / SOLUTION DDFM=Satterthwaite OUTPM=PredRICohAge; * Save for total R2;
RANDOM INTERCEPT / TYPE=UN SUBJECT=PersonID;
ODS OUTPUT InfoCrit=work.FitRICohAge CovParms=CovRICohAge;  * Save for LRT and pseudo-R2;
ESTIMATE "Total Linear Birth Cohort on Intercept"     ageT084 1 tvage84 1;
ESTIMATE "Total Quadratic Birth Cohort on Intercept"  ageT084sq 1 tvage84*ageT084 1 tvage84sq 1;
ESTIMATE "Total Linear Birth Cohort on Linear Slope"  tvage84*ageT084 1 tvage84sq 2;
RUN; TITLE1; TITLE2;
* Call macro to calculate LRT for birth cohort contextual fixed slopes;
  %FitTest(FitFewer=FitRIAge, FitMore=FitRICohAge);
* Call macro to calculate Total R2 and change;
  %TotalR2(DV=recall, Pred1=PredRIAge, Pred2=PredRICohAge);
* Call macro to calculate Pseudo-R2 and change;
  %PseudoR2(Ncov=2, CovBase=CovEmpty, CovMore1=CovRIAge, CovMore2=CovRICohAge);

TITLE1 "Model 3a Age: Add Random Linear TVage to Model 2a";
PROC MIXED DATA=work.Example3 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
CLASS PersonID;
MODEL recall = tvage84 tvage84sq ageT084 ageT084sq tvage84*ageT084
                / SOLUTION CL CHISQ DDFM=Satterthwaite;
RANDOM INTERCEPT tvage84 / GCORR TYPE=UN SUBJECT=PersonID;
ODS OUTPUT InfoCrit=work.FitRLCohAge;  * Save for LRT;
RUN; TITLE1;
* Call macro to calculate LRT for random linear TVage slope;
  %FitTest(FitFewer=FitRICohAge, FitMore=FitRLCohAge);

TITLE1 "Model 4a Age: Add Random Linear AgeCoh to Model 3a";
TITLE2 "No convergence, even after extra iterations";
PROC MIXED DATA=work.Example3 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML MAXITER=500;
CLASS PersonID;
MODEL recall = tvage84 tvage84sq ageT084 ageT084sq tvage84*ageT084
                / SOLUTION DDFM=Satterthwaite;
RANDOM INTERCEPT tvage84 ageT084 / GCORR TYPE=UN SUBJECT=PersonID;
ODS OUTPUT InfoCrit=work.FitRL2CohAge;  * Save for LRT;
RUN; TITLE1; TITLE2;
* Call macro to calculate LRT for random linear baseline age slope;
  %FitTest(FitFewer=FitRLCohAge, FitMore=FitRL2CohAge);


********************************************************************************;
******                     BEGIN TIME-IN-STUDY MODELS                    *******;
********************************************************************************;

TITLE1 "Model 1b Time: Fixed Quadratic, Random Intercept Model";
PROC MIXED DATA=work.Example3 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
CLASS PersonID;
MODEL recall = time timesq / SOLUTION DDFM=Satterthwaite OUTPM=PredRITim; * Save for total R2;
RANDOM INTERCEPT / TYPE=UN SUBJECT=PersonID;
ODS OUTPUT InfoCrit=FitRITim CovParms=CovRITim; * Save for LRT and pseudo-R2;
RUN; TITLE1;
* Call macro to calculate Total R2;
  %TotalR2(DV=recall, Pred1=PredRITim);
* Call macro to calculate Pseudo-R2;
  %PseudoR2(Ncov=2, CovBase=CovEmpty, CovMore1=CovRITim);

TITLE1 "Model 2b Time: Fixed Quadratic, Random Intercept Model";
TITLE2 "Controlling for Birth Cohort as Total Effects";
PROC MIXED DATA=work.Example3 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
CLASS PersonID;
MODEL recall = time timesq ageT084 ageT084sq time*ageT084
                / SOLUTION DDFM=Satterthwaite OUTPM=PredRICohTim; * Save for total R2;
RANDOM INTERCEPT / TYPE=UN SUBJECT=PersonID;
ODS OUTPUT InfoCrit=FitRICohTim CovParms=CovRICohTim; * Save for LRT and pseudo-R2;
ESTIMATE "Contextual Linear Birth Cohort on Intercept"     ageT084 1 time -1;
ESTIMATE "Contextual Quadratic Birth Cohort on Intercept"  ageT084sq 1 time*ageT084 -1 timesq 1;
ESTIMATE "Contextual Linear Birth Cohort on Linear Slope"  time*ageT084 1 timesq -2;
RUN; TITLE1; TITLE2;
* Call macro to calculate LRT for birth cohort between fixed slopes;
  %FitTest(FitFewer=FitRITim, FitMore=FitRICohTim);
* Call macro to calculate Total R2 and change;
  %TotalR2(DV=recall, Pred1=PredRITim, Pred2=PredRICohTim);
* Call macro to calculate Pseudo-R2 and change;
  %PseudoR2(Ncov=2, CovBase=CovEmpty, CovMore1=CovRITim, CovMore2=CovRICohTim);

TITLE1 "Model 3b Time: Add Random Linear Time to Model 2b";
PROC MIXED DATA=work.Example3 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
     CLASS PersonID;
     MODEL recall = time timesq ageT084 ageT084sq time*ageT084
                     / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time / GCORR TYPE=UN SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitRLCohTim;
RUN; TITLE1;
* Call macro to calculate LRT for random linear time slope;
  %FitTest(FitFewer=FitRICohTim, FitMore=FitRLCohTim);

TITLE1 "Model 4b Time: Add Random Linear AgeCoh to Model 3b";
TITLE2 "Converged onto NPD solution, even after extra iterations";
PROC MIXED DATA=work.Example3 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML MAXITER=500;
     CLASS PersonID;
     MODEL recall = time timesq ageT084 ageT084sq time*ageT084
                     / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time ageT084 / GCORR TYPE=UN SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitRL2CohTim;
RUN; TITLE1; TITLE2;
* Call macro to calculate LRT for random linear baseline age slope;
  %FitTest(FitFewer=FitRLCohTim, FitMore=FitRL2CohTim)


* Close external file of saved results;
ODS RTF CLOSE;

